"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsentContract = void 0;
const WrappedTransactionResponseBuilder_1 = require("../implementations/WrappedTransactionResponseBuilder");
const abi_1 = require("../interfaces/objects/abi");
const index_js_1 = require("../interfaces/objects/index.js");
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const neverthrow_result_utils_1 = require("neverthrow-result-utils");
// Note: Functions that write to the chain should return a WrappedTransactionResponse using the WrappedTransactionResponseBuilder.
let ConsentContract = class ConsentContract {
    constructor(providerOrSigner, contractAddress, cryptoUtils) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.cryptoUtils = cryptoUtils;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
            RequestForData: (ownerAddress) => {
                return this.contract.filters.RequestForData(ownerAddress);
            },
        };
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.ConsentAbi.abi, providerOrSigner);
    }
    getContractAddress() {
        return this.contractAddress;
    }
    optIn(tokenId, agreementFlags, overrides) {
        return this.writeToContract("optIn", [tokenId, agreementFlags, overrides]);
    }
    // TODO: add data permissions param
    encodeOptIn(tokenId, agreementFlags) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("optIn", [
            tokenId,
            agreementFlags,
        ]));
    }
    restrictedOptIn(tokenId, agreementFlags, signature, overrides) {
        return this.writeToContract("restrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
            overrides,
        ]);
    }
    encodeRestrictedOptIn(tokenId, signature, agreementFlags) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("restrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    anonymousRestrictedOptIn(tokenId, agreementFlags, signature, overrides) {
        return this.writeToContract("anonymousRestrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
            overrides,
        ]);
    }
    encodeAnonymousRestrictedOptIn(tokenId, signature, agreementFlags) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("anonymousRestrictedOptIn", [
            tokenId,
            agreementFlags,
            signature,
        ]));
    }
    optOut(tokenId, overrides) {
        return this.writeToContract("optOut", [tokenId, overrides]);
    }
    encodeOptOut(tokenId) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("optOut", [tokenId]));
    }
    agreementFlags(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.agreementFlagsArray(tokenId), (e) => {
            return new objects_1.ConsentContractError("Unable to call agreementFlagsArray()", e.reason, e);
        });
    }
    getMaxCapacity() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxCapacity(), (e) => {
            return new objects_1.ConsentContractError("Unable to call maxCapacity()", e.reason, e);
        }).map((bigCapacity) => {
            return bigCapacity.toNumber();
        });
    }
    updateMaxCapacity(maxCapacity, overrides) {
        return this.writeToContract("updateMaxCapacity", [maxCapacity, overrides]);
    }
    requestForData(ipfsCID, overrides) {
        return this.writeToContract("requestForData", [ipfsCID, overrides]);
    }
    // Returns the address Consent contract contract's owner
    // Note that the address on index 0 of the DEFAULT_ADMIN_ROLE members is the contract owner
    getConsentOwner() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE, 0), (e) => {
            return new objects_1.ConsentContractError("Unable to call getRoleMember()", e.reason, e);
        });
    }
    // Returns an array of address that has the DEFAULT_ADMIN_ROLE on the Consent contract
    // Note that the address on index 0 is the contract owner
    getDefaultAdminRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE), (e) => {
            return new objects_1.ConsentContractError("Unable to call getDefaultAdminRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.DEFAULT_ADMIN_ROLE, index), (e) => {
                    return new objects_1.ConsentContractError("Unable to call getRoleMember()", e.reason, e);
                });
            }));
        });
    }
    getSignerRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.SIGNER_ROLE), (e) => {
            return new objects_1.ConsentContractError("Unable to call getSignerRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.PAUSER_ROLE, index), (e) => {
                    return new objects_1.ConsentContractError("Unable to call getRoleMember() for SIGNER_ROLE", e.reason, e);
                });
            }));
        });
    }
    getPauserRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.PAUSER_ROLE), (e) => {
            return new objects_1.ConsentContractError("Unable to call getPauserRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.PAUSER_ROLE, index), (e) => {
                    return new objects_1.ConsentContractError("Unable to call getRoleMember() for PAUSER_ROLE", e.reason, e);
                });
            }));
        });
    }
    getRequesterRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(index_js_1.ConsentRoles.REQUESTER_ROLE), (e) => {
            return new objects_1.ConsentContractError("Unable to call getRequesterRoleMembers()", e.reason, e);
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(index_js_1.ConsentRoles.REQUESTER_ROLE, index), (e) => {
                    return new objects_1.ConsentContractError("Unable to call getRoleMember() for REQUESTER_ROLE", e.reason, e);
                });
            }));
        });
    }
    balanceOf(address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return new objects_1.ConsentContractError("Unable to call balanceOf()", e.reason, e);
        }).map((numberOfTokens) => {
            return numberOfTokens.toNumber();
        });
    }
    ownerOf(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.ownerOf(tokenId), (e) => {
            return new objects_1.ConsentContractError("Unable to call ownerOf()", e.reason, e);
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return new objects_1.ConsentContractError("Unable to call tokenURI()", e.reason, e);
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.ok)(null);
            }
            return (0, neverthrow_1.err)(error);
        });
    }
    queryFilter(eventFilter, fromBlock, toBlock) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryFilter(eventFilter, fromBlock, toBlock), (e) => {
            return new objects_1.ConsentContractError("Unable to call queryFilter()", e.reason, e);
        });
    }
    getConsentToken(tokenId) {
        // Get the agreement flags of the user's current consent token
        return neverthrow_result_utils_1.ResultUtils.combine([
            this.ownerOf(tokenId),
            this.agreementFlags(tokenId),
        ]).andThen(([ownerAddress, agreementFlags]) => {
            return (0, neverthrow_1.okAsync)(new objects_1.ConsentToken(this.contractAddress, ownerAddress, tokenId, new objects_1.DataPermissions(agreementFlags)));
        });
    }
    addDomain(domain, overrides) {
        return this.writeToContract("addDomain", [domain, overrides]);
    }
    removeDomain(domain, overrides) {
        return this.writeToContract("removeDomain", [domain, overrides]);
    }
    getDomains() {
        return neverthrow_1.ResultAsync.fromPromise(
        // returns array of domains
        this.contract.getDomains(), (e) => {
            return new objects_1.ConsentContractError("Unable to call getDomains()", e.reason, e);
        });
    }
    getRequestForDataListByRequesterAddress(requesterAddress, fromBlock, toBlock) {
        return this.queryFilter(this.filters.RequestForData(requesterAddress), fromBlock, toBlock).andThen((logsEvents) => {
            return neverthrow_result_utils_1.ResultUtils.combine(logsEvents.map((logEvent) => {
                var _a, _b;
                return (0, neverthrow_1.okAsync)(new objects_1.RequestForData(this.getContractAddress(), (_a = logEvent.args) === null || _a === void 0 ? void 0 : _a.requester, (_b = logEvent.args) === null || _b === void 0 ? void 0 : _b.ipfsCID, (0, objects_1.BlockNumber)(logEvent.blockNumber)));
            }));
        });
    }
    getLatestTokenIdByOptInAddress(optInAddress) {
        return this.queryFilter(this.filters.Transfer(null, optInAddress), undefined, undefined).map((logsEvents) => {
            if (logsEvents.length == 0) {
                return null;
            }
            const latestOptinEvent = logsEvents.reduce((latestEvent, logEvent) => logEvent.blockNumber > latestEvent.blockNumber
                ? logEvent
                : latestEvent, logsEvents[0]);
            if (latestOptinEvent.args && latestOptinEvent.args.tokenId) {
                return (0, objects_1.TokenId)(latestOptinEvent.args.tokenId);
            }
            return null;
        });
    }
    disableOpenOptIn(overrides) {
        return this.writeToContract("disableOpenOptIn", [overrides]);
    }
    enableOpenOptIn(overrides) {
        return this.writeToContract("enableOpenOptIn", [overrides]);
    }
    baseURI() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.baseURI(), (e) => {
            return new objects_1.ConsentContractError("Unable to call baseURI()", e.reason, e);
        });
    }
    setBaseURI(baseUri, overrides) {
        return this.writeToContract("setBaseURI", [baseUri, overrides]);
    }
    hasRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.hasRole(index_js_1.ConsentRoles[role], address), (e) => {
            return new objects_1.ConsentContractError("Unable to call hasRole()", e.reason, e);
        });
    }
    grantRole(role, address, overrides) {
        return this.writeToContract("grantRole", [role, address, overrides]);
    }
    revokeRole(role, address, overrides) {
        return this.writeToContract("revokeRole", [role, address, overrides]);
    }
    renounceRole(role, address, overrides) {
        return this.writeToContract("renounceRole", [role, address, overrides]);
    }
    getQueryHorizon() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryHorizon(), (e) => {
            return new objects_1.ConsentContractError("Unable to call queryHorizon()", e.reason, e);
        }).map((queryHorizon) => (0, objects_1.BlockNumber)(queryHorizon.toNumber()));
    }
    setQueryHorizon(blockNumber, overrides) {
        return this.writeToContract("setQueryHorizon", [blockNumber, overrides]);
    }
    // Get the number of opted in addresses
    totalSupply() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.totalSupply(), (e) => {
            return new objects_1.ConsentContractError("Unable to call totalSupply()", e.reason, e);
        }).map((totalSupply) => totalSupply.toNumber());
    }
    openOptInDisabled() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.openOptInDisabled(), (e) => {
            return new objects_1.ConsentContractError("Unable to call openOptInDisabled()", "Unknown", e);
        });
    }
    // Marketplace functions
    getMaxTags() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxTags(), (e) => {
            return new objects_1.ConsentContractError("Unable to call openOptInDisabled()", "Unknown", e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getNumberOfStakedTags() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getNumberOfStakedTags(), (e) => {
            return new objects_1.ConsentContractError("Unable to call getNumberOfStakedTags()", "Unknown", e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getTagArray() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getTagArray(), (e) => {
            return new objects_1.ConsentContractError("Unable to call getTagArray()", "Unknown", e);
        }).map((tags) => {
            return tags.map((tag) => {
                return new index_js_1.Tag(tag.slot ? (0, objects_1.BigNumberString)(tag.slot.toString()) : null, tag.tag, tag.staker);
            });
        });
    }
    newGlobalTag(tag, newStakeAmount, overrides) {
        return this.writeToContract("newGlobalTag", [
            tag,
            newStakeAmount,
            overrides,
        ]);
    }
    newLocalTagUpstream(tag, newStakeAmount, existingStakeAmount, overrides) {
        return this.writeToContract("newLocalTagUpstream", [
            tag,
            newStakeAmount,
            existingStakeAmount,
            overrides,
        ]);
    }
    newLocalTagDownstream(tag, existingStakeAmount, newStakeAmount, overrides) {
        return this.writeToContract("newLocalTagDownstream", [
            tag,
            existingStakeAmount,
            newStakeAmount,
            overrides,
        ]);
    }
    replaceExpiredListing(tag, stakeAmount, overrides) {
        return this.writeToContract("replaceExpiredListing", [
            tag,
            stakeAmount,
            overrides,
        ]);
    }
    removeListing(tag, overrides) {
        return this.writeToContract("removeListing", [tag, overrides]);
    }
    getSignature(values) {
        const types = ["address", "uint256"];
        return this.cryptoUtils.getSignature(this.providerOrSigner, types, values);
    }
    // Takes the Consent contract's function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContract(functionName, functionParams) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract[functionName](...functionParams), (e) => {
            return new objects_1.ConsentContractError(`Unable to call ${functionName}()`, e.reason, e);
        }).map((tx) => {
            return this.toWrappedTransactionResponse(tx, functionName, functionParams);
        });
    }
    toWrappedTransactionResponse(transactionResponse, functionName, functionParams) {
        var _a;
        return WrappedTransactionResponseBuilder_1.WrappedTransactionResponseBuilder.buildWrappedTransactionResponse(transactionResponse, (0, objects_1.EVMContractAddress)(this.contract.address), (0, objects_1.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, abi_1.ContractsAbis.ConsentFactoryAbi.abi);
    }
};
ConsentContract = __decorate([
    (0, inversify_1.injectable)()
], ConsentContract);
exports.ConsentContract = ConsentContract;
//# sourceMappingURL=ConsentContract.js.map