import { IConsentFactoryContract } from "../interfaces/IConsentFactoryContract";
import { ConsentRoles, ContractOverrides, WrappedTransactionResponse } from "../interfaces/objects";
import { BaseURI, BigNumberString, ConsentFactoryContractError, ConsentName, EVMAccountAddress, EVMContractAddress, MarketplaceListing, MarketplaceTag } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
export declare class ConsentFactoryContract implements IConsentFactoryContract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    protected contract: ethers.Contract;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, consentFactoryAddress: EVMContractAddress);
    getContractAddress(): EVMContractAddress;
    createConsent(ownerAddress: EVMAccountAddress, baseUri: BaseURI, name: ConsentName, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ConsentFactoryContractError>;
    getUserDeployedConsentsCount(ownerAddress: EVMAccountAddress): ResultAsync<number, ConsentFactoryContractError>;
    getUserDeployedConsentsByIndex(ownerAddress: EVMAccountAddress, startingIndex: number, endingIndex: number): ResultAsync<EVMContractAddress[], ConsentFactoryContractError>;
    getUserDeployedConsents(ownerAddress: EVMAccountAddress): ResultAsync<EVMContractAddress[], ConsentFactoryContractError>;
    getUserRoleAddressesCount(ownerAddress: EVMAccountAddress, role: ConsentRoles): ResultAsync<number, ConsentFactoryContractError>;
    getUserRoleAddressesCountByIndex(ownerAddress: EVMAccountAddress, role: ConsentRoles, startingIndex: number, endingIndex: number): ResultAsync<EVMContractAddress[], ConsentFactoryContractError>;
    getDeployedConsents(): ResultAsync<EVMContractAddress[], ConsentFactoryContractError>;
    getMaxTagsPerListing(): ResultAsync<number, ConsentFactoryContractError>;
    getListingDuration(): ResultAsync<number, ConsentFactoryContractError>;
    setListingDuration(listingDuration: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ConsentFactoryContractError>;
    setMaxTagsPerListing(maxTagsPerListing: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ConsentFactoryContractError>;
    adminRemoveListing(tag: MarketplaceTag, removedSlot: BigNumberString, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ConsentFactoryContractError>;
    getListingDetail(tag: MarketplaceTag, slot: BigNumberString): ResultAsync<MarketplaceListing, ConsentFactoryContractError>;
    getListingsForward(tag: MarketplaceTag, startingSlot: BigNumberString, numberOfSlots: number, filterActive: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError>;
    getListingsBackward(tag: MarketplaceTag, startingSlot: BigNumberString, numberOfSlots: number, filterActive: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError>;
    getTagTotal(tag: MarketplaceTag): ResultAsync<number, ConsentFactoryContractError>;
    getListingsByTag(tag: MarketplaceTag): ResultAsync<MarketplaceListing[], ConsentFactoryContractError>;
    protected writeToContract(functionName: string, functionParams: any[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ConsentFactoryContractError>;
    protected toWrappedTransactionResponse(transactionResponse: ethers.providers.TransactionResponse, functionName: string, functionParams: any[]): WrappedTransactionResponse;
}
//# sourceMappingURL=ConsentFactoryContract.d.ts.map