"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsentFactoryContract = void 0;
const WrappedTransactionResponseBuilder_1 = require("../implementations/WrappedTransactionResponseBuilder");
const abi_1 = require("../interfaces/objects/abi");
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const neverthrow_result_utils_1 = require("neverthrow-result-utils");
let ConsentFactoryContract = class ConsentFactoryContract {
    constructor(providerOrSigner, consentFactoryAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contract = new ethers_1.ethers.Contract(consentFactoryAddress, abi_1.ContractsAbis.ConsentFactoryAbi.abi, providerOrSigner);
    }
    getContractAddress() {
        var _a;
        return (0, objects_1.EVMContractAddress)(((_a = this.contract) === null || _a === void 0 ? void 0 : _a.address) || "");
    }
    createConsent(ownerAddress, baseUri, name, overrides) {
        return this.writeToContract("createConsent", [ownerAddress, baseUri, name], overrides);
    }
    // Gets the count of user's deployed Consents
    getUserDeployedConsentsCount(ownerAddress) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserDeployedConsentsCount(ownerAddress), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getUserDeployedConsentsCount()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
    // Gets the array of user deployed Consents by index count
    // Index values can be anywhere between the count obtained from getUserDeployedConsentsCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getUserDeployedConsentsByIndex()", e.reason, e);
        });
    }
    // get the latest deployed consent address by owner account address
    getUserDeployedConsents(ownerAddress) {
        return this.getUserDeployedConsentsCount(ownerAddress).andThen((count) => {
            return this.getUserDeployedConsentsByIndex(ownerAddress, 0, count);
        });
    }
    // Gets the count of Consent addresses user has specific roles for
    getUserRoleAddressesCount(ownerAddress, role) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserConsentAddressesCount(ownerAddress, role), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getUserRoleAddressesCount()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
    // Gets the array of Consent addresses user has specific roles for
    // Index values can be anywhere between the count obtained from getUserRoleAddressesCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call filters.getUserRoleAddressesCountByIndex()", e.reason, e);
        });
    }
    getDeployedConsents() {
        const eventFilter = this.contract.filters.ConsentDeployed();
        return neverthrow_1.ResultAsync.fromPromise(this.contract.queryFilter(eventFilter), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call contract.queryFilter() for ConsentDeployed event", e.reason, e);
        }).map((events) => {
            const consents = [];
            events.forEach((event) => {
                var _a;
                if ((_a = event === null || event === void 0 ? void 0 : event.args) === null || _a === void 0 ? void 0 : _a.consentAddress) {
                    consents.push((0, objects_1.EVMContractAddress)(event.args.consentAddress));
                }
            });
            return consents;
        });
    }
    // Marketplace functions
    getMaxTagsPerListing() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.maxTagsPerListing(), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getMaxTagsPerListing()", e.reason, e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    getListingDuration() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingDuration(), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getListingDuration()", e.reason, e);
        }).map((num) => {
            return num.toNumber();
        });
    }
    setListingDuration(listingDuration, overrides) {
        return this.writeToContract("setListingDuration", [listingDuration], overrides);
    }
    setMaxTagsPerListing(maxTagsPerListing, overrides) {
        return this.writeToContract("setMaxTagsPerListing", [maxTagsPerListing], overrides);
    }
    adminRemoveListing(tag, removedSlot, overrides) {
        return this.writeToContract("setMaxTagsPerListing", [tag, removedSlot], overrides);
    }
    getListingDetail(tag, slot) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListing(tag, slot), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getListing()", e.reason, e);
        }).map((listing) => {
            var _a;
            return new objects_1.MarketplaceListing((0, objects_1.BigNumberString)(listing.previous.toString()), (0, objects_1.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_1.UnixTimestamp)((_a = listing.timeExpiring) === null || _a === void 0 ? void 0 : _a.toNumber()), (0, objects_1.IpfsCID)(""), // TODO: Update contract to also return its CID for getListing (only does this with getListingsForward/backward atm)
            slot, tag);
        });
    }
    getListingsForward(tag, startingSlot, numberOfSlots, filterActive) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, filterActive), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getListingsForward()", e.reason, e);
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                return new objects_1.MarketplaceListing((0, objects_1.BigNumberString)(listing.previous.toString()), (0, objects_1.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_1.UnixTimestamp)(listing.timeExpiring.toNumber()), (0, objects_1.IpfsCID)(cids[index]), listings[index + 1] != null && listing.next.isZero() === false
                    ? (0, objects_1.BigNumberString)(listings[index + 1].previous.toString())
                    : listings[index - 1] != null
                        ? (0, objects_1.BigNumberString)(listings[index - 1].next.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    getListingsBackward(tag, startingSlot, numberOfSlots, filterActive) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, filterActive), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getListingsForward()", e.reason, e);
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                var _a;
                return new objects_1.MarketplaceListing((0, objects_1.BigNumberString)(listing.previous.toString()), (0, objects_1.BigNumberString)(listing.next.toString()), listing.consentContract, (0, objects_1.UnixTimestamp)((_a = listing.timeExpiring) === null || _a === void 0 ? void 0 : _a.toNumber()), (0, objects_1.IpfsCID)(cids[index]), listings[index + 1] != null &&
                    listing.previous.eq(ethers_1.ethers.constants.MaxUint256) === false
                    ? (0, objects_1.BigNumberString)(listings[index + 1].next.toString())
                    : listings[index - 1] != null
                        ? (0, objects_1.BigNumberString)(listings[index - 1].previous.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    getTagTotal(tag) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getTagTotal(tag), (e) => {
            return new objects_1.ConsentFactoryContractError("Unable to call getTagTotal()", e.reason, e);
        }).map((count) => {
            return count.toNumber();
        });
    }
    getListingsByTag(tag) {
        // We get the total number of slots by calling getTagTotal()
        // And if we query the 2^256 - 1 slot by calling getListingDetail(), its previous member variable will point to the highest ranked listing for that tag
        return neverthrow_result_utils_1.ResultUtils.combine([
            this.getTagTotal(tag),
            this.getListingDetail(tag, (0, objects_1.BigNumberString)(ethers_1.ethers.constants.MaxUint256.toString())),
        ]).andThen(([tagTotal, listingDetail]) => {
            // The max slot's next points to the highest slot
            const highestRankListingSlot = listingDetail.next;
            // Fetch from the highest to lowest listings
            return this.getListingsForward(tag, highestRankListingSlot, tagTotal, true);
        });
    }
    // Takes the Consent contract's function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContract(functionName, functionParams, overrides) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return new objects_1.ConsentFactoryContractError(`Unable to call ${functionName}()`, e.reason, e);
        }).map((tx) => {
            return this.toWrappedTransactionResponse(tx, functionName, functionParams);
        });
    }
    toWrappedTransactionResponse(transactionResponse, functionName, functionParams) {
        var _a;
        return WrappedTransactionResponseBuilder_1.WrappedTransactionResponseBuilder.buildWrappedTransactionResponse(transactionResponse, (0, objects_1.EVMContractAddress)(this.contract.address), (0, objects_1.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, abi_1.ContractsAbis.ConsentFactoryAbi.abi);
    }
};
ConsentFactoryContract = __decorate([
    (0, inversify_1.injectable)()
], ConsentFactoryContract);
exports.ConsentFactoryContract = ConsentFactoryContract;
// I listingStruct { at the place where we're using it, and don't have to export here
// Alternative option is to get the deployed Consent addresses through filtering event ConsentDeployed() event
/* // TODO: Replace Promise<any> with correct types returned from ConsentDeployed() and queryFilter()
  public getConsentsDeployedByOwner(
    ownerAddress: EVMAccountAddress,
  ): ResultAsync<EVMContractAddress[], ConsentFactoryContractError> {
    return ResultAsync.fromPromise(
      this.contract.filters.ConsentDeployed(ownerAddress) as Promise<any>,
      (e) => {
        return new ConsentFactoryContractError(
          "Unable to call filters.ConsentDeployed()",
          (e as IBlockchainError).reason,
          e,
        );
      },
    )
      .andThen((_logs) => {
        return ResultAsync.fromPromise(
          this.contract.queryFilter(_logs) as Promise<any>,
          (e) => {
            return new ConsentFactoryContractError(
              "Unable to call filters.ConsentDeployed()",
              (e as IBlockchainError).reason,
              e,
            );
          },
        );
      })
      .map((logs) => {
        return logs.map((log) => log.args.consentAddress);
      });
  }
} */
//# sourceMappingURL=ConsentFactoryContract.js.map