import { ICrumbsContract } from "../interfaces/ICrumbsContract";
import { ContractOverrides } from "../interfaces/objects/ContractOverrides";
import { EVMAccountAddress, EVMContractAddress, TokenUri, TokenId, CrumbsContractError, HexString } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { WrappedTransactionResponse } from "..";
export declare class CrumbsContract implements ICrumbsContract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    contractAddress: EVMContractAddress;
    protected contract: ethers.Contract;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, contractAddress: EVMContractAddress);
    addressToCrumbId(accountAddress: EVMAccountAddress): ResultAsync<TokenId | null, CrumbsContractError>;
    tokenURI(tokenId: TokenId): ResultAsync<TokenUri | null, CrumbsContractError>;
    createCrumb(crumbId: TokenId, tokenUri: TokenUri, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, CrumbsContractError>;
    encodeCreateCrumb(crumbId: TokenId, crumbContent: TokenUri): HexString;
    burnCrumb(crumbId: TokenId, overrides?: ContractOverrides | undefined): ResultAsync<WrappedTransactionResponse, CrumbsContractError>;
    encodeBurnCrumb(crumbId: TokenId): HexString;
    updateTokenURI(crumbId: TokenId, tokenURI: TokenUri, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, CrumbsContractError>;
    getContract(): ethers.Contract;
    protected writeToContract(functionName: string, functionParams: any[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, CrumbsContractError>;
    protected toWrappedTransactionResponse(transactionResponse: ethers.providers.TransactionResponse, functionName: string, functionParams: any[]): WrappedTransactionResponse;
}
//# sourceMappingURL=CrumbsContract.d.ts.map