import { ERewardRoles } from "../interfaces/enums/ERewardRoles.js";
import { ContractOverrides, IERC721RewardContract, WrappedTransactionResponse } from "../interfaces/index.js";
import { EVMAccountAddress, EVMContractAddress, TokenUri, TokenId, BaseURI, ERC721RewardContractError } from "@snickerdoodlelabs/objects";
import { ethers, EventFilter } from "ethers";
import { ResultAsync } from "neverthrow";
export declare class ERC721RewardContract implements IERC721RewardContract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    contractAddress: EVMContractAddress;
    protected contract: ethers.Contract;
    protected contractFactory: ethers.ContractFactory;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, contractAddress: EVMContractAddress);
    getContractAddress(): EVMContractAddress;
    getOwner(): ResultAsync<EVMAccountAddress, ERC721RewardContractError>;
    getDefaultAdminRoleMembers(): ResultAsync<EVMAccountAddress[], ERC721RewardContractError>;
    getMinterRoleMembers(): ResultAsync<EVMAccountAddress[], ERC721RewardContractError>;
    baseURI(): ResultAsync<BaseURI, ERC721RewardContractError>;
    setBaseURI(baseUri: BaseURI, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ERC721RewardContractError>;
    balanceOf(address: EVMAccountAddress): ResultAsync<number, ERC721RewardContractError>;
    ownerOf(tokenId: TokenId): ResultAsync<EVMAccountAddress, ERC721RewardContractError>;
    tokenURI(tokenId: TokenId): ResultAsync<TokenUri | null, ERC721RewardContractError>;
    hasRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress): ResultAsync<boolean, ERC721RewardContractError>;
    grantRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ERC721RewardContractError>;
    revokeRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ERC721RewardContractError>;
    renounceRole(role: keyof typeof ERewardRoles, address: EVMAccountAddress, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ERC721RewardContractError>;
    filters: {
        Transfer: (fromAddress: EVMAccountAddress | null, toAddress: EVMAccountAddress | null) => EventFilter;
    };
    getContract(): ethers.Contract;
    protected writeToContract(functionName: string, functionParams: any[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, ERC721RewardContractError>;
    protected toWrappedTransactionResponse(transactionResponse: ethers.providers.TransactionResponse, functionName: string, functionParams: any[]): WrappedTransactionResponse;
}
//# sourceMappingURL=ERC721RewardContract.d.ts.map