"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RewardsContractFactory = void 0;
const GasUtils_1 = require("../implementations/GasUtils");
const WrappedTransactionResponseBuilder_1 = require("../implementations/WrappedTransactionResponseBuilder");
const objects_1 = require("../interfaces/objects");
const objects_2 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
let RewardsContractFactory = class RewardsContractFactory {
    constructor(providerOrSigner, rewardType) {
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.contractFactory = new ethers_1.ethers.ContractFactory(objects_1.ContractsAbis.ERC721Reward.abi, objects_1.ContractsAbis.ERC721Reward.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract
    deployERC721Reward(name, symbol, baseURI, overrides) {
        return GasUtils_1.GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            const contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            return this.writeToContract("deploy", [name, symbol, baseURI], contractOverrides, true);
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return neverthrow_1.ResultAsync.fromPromise(this.providerOrSigner.estimateGas(this.contractFactory.getDeployTransaction(name, symbol, baseURI)), (e) => {
            return new objects_2.RewardsFactoryError("Failed to wait() for contract deployment", e.reason, e);
        }).map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return estimatedGas.mul(120).div(100);
        });
    }
    // Takes the ERC721 factory's function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContract(functionName, functionParams, overrides, isDeployingContract) {
        // If we are deploying a contract, the deploy() call returns an ethers.Contract object and the txresponse is under the deployTransaction property
        if (isDeployingContract == true) {
            return neverthrow_1.ResultAsync.fromPromise(this.contractFactory[functionName](...functionParams, {
                ...overrides,
            }), (e) => {
                return new objects_2.RewardsFactoryError(`Unable to call ${functionName}()`, e.reason, e);
            }).map((tx) => {
                return this.toWrappedTransactionResponse(tx.deployTransaction, functionName, functionParams);
            });
        }
        // If we are not deploying a contract, the contract function call returns the txresponse
        return neverthrow_1.ResultAsync.fromPromise(this.contractFactory[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return new objects_2.RewardsFactoryError(`Unable to call ${functionName}()`, e.reason, e);
        }).map((tx) => {
            return this.toWrappedTransactionResponse(tx, functionName, functionParams);
        });
    }
    toWrappedTransactionResponse(transactionResponse, functionName, functionParams) {
        var _a;
        return WrappedTransactionResponseBuilder_1.WrappedTransactionResponseBuilder.buildWrappedTransactionResponse(transactionResponse, (0, objects_2.EVMContractAddress)(""), (0, objects_2.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, objects_1.ContractsAbis.ConsentFactoryAbi.abi);
    }
};
RewardsContractFactory = __decorate([
    (0, inversify_1.injectable)()
], RewardsContractFactory);
exports.RewardsContractFactory = RewardsContractFactory;
//# sourceMappingURL=RewardsContractFactory.js.map