"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiftContract = void 0;
const WrappedTransactionResponseBuilder_1 = require("../implementations/WrappedTransactionResponseBuilder");
const abi_1 = require("../interfaces/objects/abi");
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
let SiftContract = class SiftContract {
    constructor(providerOrSigner, contractAddress) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi_1.ContractsAbis.SiftAbi.abi, providerOrSigner);
    }
    checkURL(domain) {
        // Returns the tokenURI or string
        // eg. 'www.sift.com/VERIFIED', 'www.sift.com/MALICIOUS' or 'NOT VERIFIED'
        return neverthrow_1.ResultAsync.fromPromise(this.contract.checkURL(domain), (e) => {
            return new objects_1.SiftContractError("Unable to call checkURL()", e.reason, e);
        });
    }
    verifyURL(domain, overrides) {
        return this.writeToContract("verifyURL", [domain], overrides);
    }
    maliciousURL(domain, overrides) {
        return this.writeToContract("maliciousURL", [domain], overrides);
    }
    setBaseURI(baseUri, overrides) {
        return this.writeToContract("setBaseURI", [baseUri], overrides);
    }
    getContract() {
        return this.contract;
    }
    // Takes the Sift contract's function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContract(functionName, functionParams, overrides) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return new objects_1.SiftContractError(`Unable to call ${functionName}()`, e.reason, e);
        }).map((tx) => {
            return this.toWrappedTransactionResponse(tx, functionName, functionParams);
        });
    }
    toWrappedTransactionResponse(transactionResponse, functionName, functionParams) {
        var _a;
        return WrappedTransactionResponseBuilder_1.WrappedTransactionResponseBuilder.buildWrappedTransactionResponse(transactionResponse, (0, objects_1.EVMContractAddress)(this.contract.address), (0, objects_1.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, abi_1.ContractsAbis.ConsentFactoryAbi.abi);
    }
};
SiftContract = __decorate([
    (0, inversify_1.injectable)()
], SiftContract);
exports.SiftContract = SiftContract;
//# sourceMappingURL=SiftContract.js.map